"""
@author: caddiesnew
@file: moorethread.py
@time: 2025-05-20 16:39:17
@description: MooreThread GPU信息采集
"""

import json
import math
import platform
import subprocess
from typing import Optional, Tuple

from ..type import HardwareCollector as H
from ..type import HardwareConfig, HardwareFuncResult, HardwareInfoList
from ..utils import generate_key, random_index


def get_moorethreads_gpu_info() -> HardwareFuncResult:
    """
    获取MooreThreads GPU信息，包括驱动版本、设备信息等
    """
    # MooreThreads GPU只支持Linux系统
    if platform.system() != "Linux":
        return None, None

    info = {"driver": None, "gpu": None}
    collector = None
    try:
        driver, gpu_map = map_moorethreads_gpu()
        info["driver"] = driver
        info["gpu"] = gpu_map
        max_mem_value = 0
        for gpu_id in gpu_map:
            mem_value = int(gpu_map[gpu_id]["memory"])
            max_mem_value = max(max_mem_value, mem_value)
        max_mem_value *= 1024
        collector = MTTCollector(gpu_map, max_mem_value)
    except Exception:  # noqa
        if all(v is None for v in info.values()):
            return None, None
    return info, collector


def map_moorethreads_gpu() -> Tuple[Optional[str], dict]:
    """
    获取 Moore Threads GPU信息，包括驱动版本、设备信息等，例如：
    driver: '2.7.0'
    gpu_map: {"0": { "name": "MTT S4000”, "memory": "48"}, "1": { "name": "MTT S4000", "memory": "48"}, ...}
    """
    output_str = subprocess.run(
        ["mthreads-gmi", "-q", "--json"], capture_output=True, check=True, text=True
    ).stdout
    output_json = json.loads(output_str)
    driver = None
    gpu_map = {}

    if "Driver Version" in output_json.keys():
        driver = output_json["Driver Version"]

    for gpu_info in output_json["GPU"]:
        gpu_id = gpu_info["Index"]
        gpu_name = gpu_info["Product Name"]
        gpu_memory = gpu_info["FB Memory Usage"]["Total"]
        if gpu_memory.endswith("MiB"):
            total = int(gpu_memory[:-3]) // 1024
            gpu_memory = str(total)
        elif gpu_memory.endswith("GiB"):
            total = int(gpu_memory[:-3])
            gpu_memory = str(total)
        else:
            pass
        gpu_map[gpu_id] = {
            "name": gpu_name,
            "memory": gpu_memory,
        }
    return driver, gpu_map


class MTTCollector(H):
    def __init__(self, gpu_map, max_mem_value):
        super().__init__()
        self.gpu_map = gpu_map
        self.max_mem_value = max_mem_value

        # GPU Utilization (%)
        self.util_key = generate_key("gpu.{gpu_index}.pct")
        util_config = HardwareConfig(
            y_range=(0, 100),
            chart_index=random_index(),
            chart_name="GPU Utilization (%)",
        )
        self.per_util_configs = {}

        # GPU Memory Allocated (%)
        self.memory_key = generate_key("gpu.{gpu_index}.mem.pct")
        memory_config = HardwareConfig(
            y_range=(0, 100),
            chart_index=random_index(),
            chart_name="GPU Memory Allocated (%)",
        )
        self.per_memory_configs = {}

        # GPU Memory Allocated (MB)
        self.mem_value_key = generate_key("gpu.{gpu_index}.mem.value")
        mem_value_config = HardwareConfig(
            y_range=(0, self.max_mem_value),
            chart_index=random_index(),
            chart_name="GPU Memory Allocated (MB)",
        )
        self.per_mem_value_configs = {}

        # GPU Temperature (°C)
        self.temp_key = generate_key("gpu.{gpu_index}.temp")
        temp_config = HardwareConfig(
            y_range=(0, None),
            chart_index=random_index(),
            chart_name="GPU Temperature (°C)",
        )
        self.per_temp_configs = {}

        # GPU Power (W)
        self.power_key = generate_key("gpu.{gpu_index}.power")
        power_config = HardwareConfig(
            y_range=(0, None),
            chart_index=random_index(),
            chart_name="GPU Power (W)",
        )
        self.per_power_configs = {}

        for gpu_id in self.gpu_map:
            metric_name = f"GPU {gpu_id}"
            self.per_util_configs[metric_name] = util_config.clone(
                metric_name=metric_name
            )
            self.per_memory_configs[metric_name] = memory_config.clone(
                metric_name=metric_name
            )
            self.per_mem_value_configs[metric_name] = mem_value_config.clone(
                metric_name=metric_name
            )
            self.per_temp_configs[metric_name] = temp_config.clone(
                metric_name=metric_name
            )
            self.per_power_configs[metric_name] = power_config.clone(
                metric_name=metric_name
            )

    def collect(self) -> HardwareInfoList:
        result: HardwareInfoList = []
        usage_methods = [
            self.get_utilization_usage,
            self.get_memory_usage,
            self.get_mem_value_usage,
            self.get_temperature_usage,
            self.get_power_usage,
        ]

        for method in usage_methods:
            result.extend(method().values())
        return result

    def get_utilization_usage(self) -> dict:
        """
        获取指定GPU设备的利用率
        """
        output_str = subprocess.run(
            ["mthreads-gmi", "-q", "-d", "UTILIZATION", "--json"],
            capture_output=True,
            text=True,
        ).stdout
        output_json = json.loads(output_str)
        util_infos = {}
        for gpu_info in output_json["GPU"]:
            gpu_id = gpu_info["Index"]

            util_infos[gpu_id] = {
                "key": self.util_key.format(gpu_index=gpu_id),
                "name": f"GPU {gpu_id} Utilization (%)",
                "value": math.nan,
                "config": self.per_util_configs[f"GPU {gpu_id}"],
            }
            # 解析利用率
            gpu_utilization = gpu_info["Utilization"]["Gpu"]

            if isinstance(gpu_utilization, str) and gpu_utilization.endswith("%"):
                gpu_utilization = gpu_utilization[:-1]
            try:
                util_infos[gpu_id]["value"] = float(gpu_utilization)
            except ValueError:
                pass
        return util_infos

    def get_memory_usage(self) -> dict:
        """
        获取指定GPU设备的内存占用率
        """
        output_str = subprocess.run(
            ["mthreads-gmi", "-q", "-d", "MEMORY", "--json"],
            capture_output=True,
            text=True,
        ).stdout
        output_json = json.loads(output_str)
        mem_infos = {}
        for gpu_info in output_json["GPU"]:
            gpu_id = gpu_info["Index"]

            mem_infos[gpu_id] = {
                "key": self.memory_key.format(gpu_index=gpu_id),
                "name": f"GPU {gpu_id} Memory Allocated (%)",
                "value": math.nan,
                "config": self.per_memory_configs[f"GPU {gpu_id}"],
            }
            gpu_mem_info = gpu_info["FB Memory Usage"]
            gpu_mem_toal = gpu_mem_info["Total"].replace("MiB", "").strip()
            gpu_mem_used = gpu_mem_info["Used"].replace("MiB", "").strip()
            gpu_mem_usage_rate = float(gpu_mem_used) / float(gpu_mem_toal) * 100
            mem_infos[gpu_id]["value"] = gpu_mem_usage_rate
        return mem_infos

    def get_mem_value_usage(self) -> dict:
        """
        获取指定GPU设备的显存使用量
        """
        output_str = subprocess.run(
            ["mthreads-gmi", "-q", "-d", "MEMORY", "--json"],
            capture_output=True,
            text=True,
        ).stdout
        output_json = json.loads(output_str)
        mem_infos = {}
        for gpu_info in output_json["GPU"]:
            gpu_id = gpu_info["Index"]

            mem_infos[gpu_id] = {
                "key": self.mem_value_key.format(gpu_index=gpu_id),
                "name": f"GPU {gpu_id} Memory Allocated (MB)",
                "value": math.nan,
                "config": self.per_mem_value_configs[f"GPU {gpu_id}"],
            }
            gpu_mem_info = gpu_info["FB Memory Usage"]
            mem_infos[gpu_id]["value"] = float(
                gpu_mem_info["Used"].replace("MiB", "").strip()
            )
        return mem_infos

    def get_temperature_usage(self) -> dict:
        """
        获取指定GPU设备的温度(°C)
        """
        output_str = subprocess.run(
            ["mthreads-gmi", "-q", "-d", "TEMPERATURE", "--json"],
            capture_output=True,
            text=True,
        ).stdout
        output_json = json.loads(output_str)
        temp_infos = {}
        for gpu_info in output_json["GPU"]:
            gpu_id = gpu_info["Index"]

            temp_infos[gpu_id] = {
                "key": self.temp_key.format(gpu_index=gpu_id),
                "name": f"GPU {gpu_id} Temperature (°C)",
                "value": math.nan,
                "config": self.per_temp_configs[f"GPU {gpu_id}"],
            }
            # 解析温度
            gpu_temp = (
                gpu_info["Temperature"]["GPU Current Temp"].replace("C", "").strip()
            )
            temp_infos[gpu_id]["value"] = float(gpu_temp)
        return temp_infos

    def get_power_usage(self) -> dict:
        """
        获取指定GPU设备的功耗(W)
        """
        output_str = subprocess.run(
            ["mthreads-gmi", "-q", "-d", "POWER", "--json"],
            capture_output=True,
            text=True,
        ).stdout
        output_json = json.loads(output_str)
        power_infos = {}
        for gpu_info in output_json["GPU"]:
            gpu_id = gpu_info["Index"]

            power_infos[gpu_id] = {
                "key": self.power_key.format(gpu_index=gpu_id),
                "name": f"GPU {gpu_id} Power (W)",
                "value": math.nan,
                "config": self.per_power_configs[f"GPU {gpu_id}"],
            }
            gpu_power = (
                gpu_info["Power Readings"]["Power Draw "].strip().replace("W", "")
            )
            power_infos[gpu_id]["value"] = float(gpu_power)
        return power_infos



from pathlib import Path
from typing import Any, Callable, Dict, List, Optional, Type, Union

from swanlab.toolkit import MediaType
from .model3d import Model3D
from .molecule import Molecule
from .point_cloud import Box, PointCloud

try:
    # noinspection PyPackageRequirements
    import numpy as np

    ndarray = np.ndarray
except ImportError:
    np = None

    ndarray = None

try:
    from rdkit.Chem import Mol
except ImportError:
    Mol = None


class Object3D:
    """A dispatcher class that converts different types of 3D data to MediaType objects.

    :doc: `Object3D API Reference <https://docs.swanlab.cn/api/py-object3d.html>`

    This class provides a unified interface for handling various 3D data formats including:
    - Point clouds from numpy arrays
    - Point clouds with boxes from {"points": ndarray, "boxes": List[Box]}
    - 3D models from files (.glb)
    - Point cloud files (.swanlab.pts.json)

    Usage Examples:
    --------------
    1. Creating from numpy array (point cloud):
        >>> import numpy as np
        >>> # XYZ coordinates (N, 3)
        >>> points_xyz = np.random.rand(100, 3)
        >>> obj1 = Object3D(points_xyz)
        >>>
        >>> # XYZC coordinates with category (N, 4)
        >>> points_xyzc = np.random.rand(100, 4)
        >>> obj2 = Object3D(points_xyzc)
        >>>
        >>> # XYZRGB coordinates with colors (N, 6)
        >>> points_xyzrgb = np.random.rand(100, 6)
        >>> obj3 = Object3D(points_xyzrgb, caption="Colored Points")

    2. Creating from files:
        >>> # From GLB file
        >>> obj4 = Object3D("model.glb")
        >>>
        >>> # From SwanLab point cloud file
        >>> obj5 = Object3D("points.swanlab.pts.json")

    3. With optional parameters:
        >>> obj6 = Object3D(
        ...     points_xyz,
        ...     caption="My 3D"   # Caption text
        ... )

    4. Creating PointCloud With boxes:
        >>> obj7 = Object3D(
        ...     {"points": points_xyz, "boxes": list(Box)}
        ... )

    5. Creating from Molecule:
        >>> from rdkit import Chem
        >>> mol = Chem.MolFromSmiles("CCO")
        >>> molecule = Molecule.from_mol(mol, caption="Ethanol")
        >>> obj8 = Object3D(molecule)

    Args:
        data: Input data, can be:
            - numpy.ndarray: Point cloud data with shape (N, C) where C is 3,4 or 6
            - str/Path: Path to a 3D file (.glb or .swanlab.pts.json)
            - Molecule: A Molecule object
        caption: Optional description text
        **kwargs: Additional keyword arguments passed to specific handlers

    Returns:
        MediaType: A MediaType object (PointCloud or Model3D)

    Raises:
        ImportError: If numpy is not installed
        ValueError: If input format is not supported
        FileNotFoundError: If input file does not exist
        TypeError: If input type is not supported
    """

    def __new__(
        cls,
        data: Union[ndarray, str, Path, Dict, Mol],
        *,
        caption: Optional[str] = None,
        **kwargs,
    ) -> MediaType:
        cls._check_numpy()

        kwargs['caption'] = caption

        if isinstance(data, ndarray):
            return cls._handle_ndarray(data, **kwargs)

        if isinstance(data, (str, Path)):
            return cls._handle_file(Path(data), **kwargs)

        if isinstance(data, Mol):
            return Molecule.from_mol(data, **kwargs)

        return cls._handle_data(data, **kwargs)

    @staticmethod
    def _check_numpy() -> None:
        if np is None:
            raise ImportError("Numpy is required for Object3D class. Please install it with: pip install numpy.")

    @classmethod
    def _handle_ndarray(cls, data: ndarray, **kwargs) -> MediaType:
        point_cloud_channel_handlers = {
            3: [PointCloud.from_xyz],  # xyz
            4: [PointCloud.from_xyzc],  # xyzc
            6: [PointCloud.from_xyzrgb],  # xyzrgb
        }

        if data.ndim == 2 and data.shape[1] in point_cloud_channel_handlers:
            return cls._try_all(point_cloud_channel_handlers[data.shape[1]], data, **kwargs)

        raise ValueError(
            f"Unsupported array format: shape={data.shape}. "
            f"Expected 2D array with {list(point_cloud_channel_handlers.keys())} channels"
        )

    _FILE_HANDLERS: Dict[str, List[Callable]] = {
        '.swanlab.pts.json': [PointCloud.from_swanlab_pts_json_file],
        '.glb': [Model3D.from_glb_file],
        '.sd': [Molecule.from_sdf_file],
        '.sdf': [Molecule.from_sdf_file],
        '.mol': [Molecule.from_mol_file],
        '.pdb': [Molecule.from_pdb_file],
    }

    @classmethod
    def _handle_file(cls, path: Path, **kwargs) -> MediaType:
        if not path.exists():
            raise FileNotFoundError(f"File not found: {path}")

        if not path.is_file():
            raise ValueError(f"Path is not a file: {path}")

        suffixes = path.suffixes  # 例如: ['.xyz', '.pts', '.json']
        if not suffixes:
            raise ValueError(f"File has no extension name: {path}")

        all_tried_suffixes = [''.join(suffixes[i:]).lower() for i in range(len(suffixes))]

        for suffix in all_tried_suffixes:
            handler = cls._FILE_HANDLERS.get(suffix)
            if handler is not None:
                try:
                    return cls._try_all(handler, path, **kwargs)
                except Exception as e:
                    raise ValueError(
                        f"Error processing file {path} with handler for {suffix}:\n{str(e)}",
                    ) from e

        raise ValueError(
            f"Unsupported file type: {path.name}\n"
            f"File extension: {''.join(suffixes)}\n"
            f"Tried extensions: {', '.join(all_tried_suffixes)}\n"
            f"Supported extensions: {', '.join(sorted(cls._FILE_HANDLERS.keys()))}"
        )

    _TYPE_HANDLERS: Dict[Type, List[Callable]] = {
        dict: [PointCloud.from_swanlab_pts],
    }

    @classmethod
    def _handle_data(cls, data: Any, **kwargs) -> MediaType:
        handler = cls._TYPE_HANDLERS.get(type(data))

        if handler is not None:
            return cls._try_all(handler, data, **kwargs)

        raise TypeError(f"Unsupported input type: {type(data)}")

    @classmethod
    def _try_all(cls, handlers: List[Callable], data: Any, **kwargs) -> MediaType:
        errors = []

        for handler in handlers:
            try:
                return handler(data, **kwargs)
            except Exception as e:
                errors.append(f"Error processing data with handler {handler.__name__}: {str(e)}")

        error_message = ';\n'.join(errors)
        raise ValueError(f"All handlers failed:\n{error_message}")

    @classmethod
    def from_point_data(
        cls,
        points: ndarray,
        *,
        boxes: Optional[List[Box]] = None,
        caption: Optional[str] = None,
        **kwargs,
    ):
        return PointCloud.from_swanlab_pts(
            {
                'points': points,
                'boxes': boxes,
            },
            caption=caption,
            **kwargs,
        )



import time

import psutil

from swanlab.data.run.metadata.hardware.network import (
    NetworkCollector,
)


class TestNetworkCollector:
    collector = NetworkCollector()

    def test_network_sent(self):
        """
        测试网络信息采集
        """
        start = time.time()
        current_net_io = psutil.net_io_counters()
        time.sleep(1)

        result = self.collector.get_network_sent_speed(current_net_io.bytes_sent, time.time() - start)
        assert result['name'] == "Network Traffic Sent (KB)"
        assert result['value'] >= 0
        assert result['config'].chart_name == 'Network Traffic (KB)'

    def test_network_recv(self):
        """
        测试网络信息采集
        """
        start = time.time()
        current_net_io = psutil.net_io_counters()
        time.sleep(1)

        result = self.collector.get_network_recv_speed(current_net_io.bytes_recv, time.time() - start)
        assert result['name'] == "Network Traffic Received (KB)"
        assert result['value'] >= 0
        assert result['config'].chart_name == 'Network Traffic (KB)'



import torch.nn.functional as F
import torchvision
import torchvision.transforms as transforms
from torch.optim import SGD
from torch.utils.data import DataLoader

from mmengine.evaluator import BaseMetric
from mmengine.model import BaseModel
from mmengine.runner import Runner
from mmengine.visualization import Visualizer


from tutils import open_dev_mode
import swanlab
from swanlab.integration.mmengine import SwanlabVisBackend

swanlab.login(open_dev_mode())


class MMResNet50(BaseModel):
    def __init__(self):
        super().__init__()
        self.resnet = torchvision.models.resnet50()

    def forward(self, imgs, labels, mode):
        x = self.resnet(imgs)
        if mode == "loss":
            return {"loss": F.cross_entropy(x, labels)}
        elif mode == "predict":
            return x, labels


class Accuracy(BaseMetric):
    def process(self, data_batch, data_samples):
        score, gt = data_samples
        self.results.append(
            {
                "batch_size": len(gt),
                "correct": (score.argmax(dim=1) == gt).sum().cpu(),
            }
        )

    def compute_metrics(self, results):
        total_correct = sum(item["correct"] for item in results)
        total_size = sum(item["batch_size"] for item in results)
        return dict(accuracy=100 * total_correct / total_size)


norm_cfg = dict(mean=[0.491, 0.482, 0.447], std=[0.202, 0.199, 0.201])
train_dataloader = DataLoader(
    batch_size=32,
    shuffle=True,
    dataset=torchvision.datasets.CIFAR10(
        "data/cifar10",
        train=True,
        download=True,
        transform=transforms.Compose(
            [
                transforms.RandomCrop(32, padding=4),
                transforms.RandomHorizontalFlip(),
                transforms.ToTensor(),
                transforms.Normalize(**norm_cfg),
            ]
        ),
    ),
)

val_dataloader = DataLoader(
    batch_size=32,
    shuffle=False,
    dataset=torchvision.datasets.CIFAR10(
        "data/cifar10",
        train=False,
        download=True,
        transform=transforms.Compose([transforms.ToTensor(), transforms.Normalize(**norm_cfg)]),
    ),
)

visualizer = Visualizer(
    vis_backends=SwanlabVisBackend(init_kwargs={})
)  # init args can be found in https://docs.swanlab.cn/zh/guide_cloud/integration/integration-mmengine.html

runner = Runner(
    model=MMResNet50(),
    work_dir="./work_dir",
    train_dataloader=train_dataloader,
    optim_wrapper=dict(optimizer=dict(type=SGD, lr=0.001, momentum=0.9)),
    train_cfg=dict(by_epoch=True, max_epochs=5, val_interval=1),
    val_dataloader=val_dataloader,
    val_cfg=dict(),
    val_evaluator=dict(type=Accuracy),
    visualizer=visualizer,
)
runner.train()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/14 22:25
@File: test_types.py
@IDE: pycharm
@Description:
    测试开放API模型定义
"""

from swanlab.api.types import Experiment


def test_dict_style_access():
    """
    测试字典风格访问对象字段
    """
    exp = Experiment.model_validate({
        "cuid": "test_cuid",
        "name": "test_experiment",
        "description": "test_description",
        "state": "FINISHED",
        "show": True,
        "createdAt": "2025-05-14T22:25:00Z",
        "finishedAt": "2025-05-14T22:30:00Z",
        "user": {
            "username": "test_user",
            "name": "Test User"
        },
        "profile": {
            "requirements": "test_requirements",
        }
    })
    assert exp["cuid"] == "test_cuid"
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 00:51
@File: __init__.py.py
@IDE: pycharm
@Description:
    转换部分，兼容其他可视化工具，转换为swanlab格式
"""


# 使用延迟导入的方式
def __getattr__(name):
    if name == "TFBConverter":
        from .tfb import TFBConverter

        return TFBConverter
    if name == "WandbConverter":
        from .wb import WandbConverter

        return WandbConverter
    
    if name == "WandbLocalConverter":
        from .wb import WandbLocalConverter

        return WandbLocalConverter
    
    if name == "MLFlowConverter":
        from .mlf import MLFLowConverter
        
        return MLFLowConverter

    raise AttributeError(f"module 'convert' has no attribute '{name}'")


__all__ = ["TFBConverter", "WandbConverter", "WandbLocalConverter", "MLFlowConverter"]



"""
@author: cunyue
@file: disk.py
@time: 2024/12/3 20:12
@description: 磁盘信息采集
"""

import time
from typing import List

import psutil

from swanlab.swanlab_settings import get_settings
from .type import HardwareFuncResult, HardwareCollector, HardwareInfo, HardwareConfig
from .utils import random_index, generate_key


def get_disk_info() -> HardwareFuncResult:
    """获取磁盘信息"""
    return None, DiskCollector()


class DiskCollector(HardwareCollector):
    def __init__(self):
        super().__init__()
        self.last_disk_io = psutil.disk_io_counters()
        self.last_time = time.time()

        self.disk_path = get_settings().disk_io_dir

        disk_base_config = HardwareConfig(
            y_range=(0, None),
            chart_name="Disk I/O Utilization (MB)",
            chart_index=random_index(),
        ).clone()

        # 磁盘读取速度
        self.read_key = generate_key("disk.read")
        self.read_config = disk_base_config.clone(metric_name="read")

        # 磁盘写入速度
        self.write_key = generate_key("disk.write")
        self.write_config = disk_base_config.clone(metric_name="write")

        # 磁盘使用率
        self.usage_key = generate_key("disk.usage")
        self.usage_config = HardwareConfig(
            y_range=(0, 100),
            chart_name="Disk Utilization (%)",
        ).clone()

        self.unit = 1024**2

    def get_disk_read_speed(self, read_bytes: int, time_diff: float) -> HardwareInfo:
        """
        获取磁盘读取速度 (MB/s)
        """
        diff = read_bytes - self.last_disk_io.read_bytes
        return {
            "key": self.read_key,
            "name": "MB read from disk",
            "value": self.division_guard(diff, time_diff * self.unit),
            "config": self.read_config,
        }

    def get_disk_write_speed(self, write_bytes: int, time_diff: float) -> HardwareInfo:
        """
        获取磁盘写入速度 (MB/s)
        """
        diff = write_bytes - self.last_disk_io.write_bytes
        return {
            "key": self.write_key,
            "name": "MB written to disk",
            "value": self.division_guard(diff, time_diff * self.unit),
            "config": self.write_config,
        }

    def get_disk_usage(self) -> HardwareInfo:
        """
        获取磁盘使用率
        """
        result = psutil.disk_usage(self.disk_path).percent
        return {
            "key": self.usage_key,
            "name": "Disk Utilization (%)",
            "value": result,
            "config": self.usage_config,
        }

    def collect(self) -> List[HardwareInfo]:
        current_disk_io = psutil.disk_io_counters()
        current_time = time.time()
        time_diff = current_time - self.last_time

        disk_result = [
            self.get_disk_read_speed(current_disk_io.read_bytes, time_diff),
            self.get_disk_write_speed(current_disk_io.write_bytes, time_diff),
            self.get_disk_usage(),
        ]
        # 更新上次的值
        self.last_disk_io = current_disk_io
        self.last_time = current_time
        return disk_result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/3 15:44
@File: pytest_text.py
@IDE: pycharm
@Description:
    测试文本处理模块
"""
import pytest

from swanlab.data.modules import Text


def test_text_ok():
    # ---------------------------------- 字符串输入 ----------------------------------
    mock = "这是一段测试文本"
    text = Text(data=mock)
    data, buffer = text.parse()
    assert data == mock
    assert buffer is None
    assert text.get_more() is None
    # ---------------------------------- float输入 ----------------------------------
    mock = 1.0
    text = Text(data=mock)
    data, buffer = text.parse()
    assert data == "1.0"
    assert buffer is None
    assert text.get_more() is None
    # ---------------------------------- int输入 ----------------------------------
    mock = 1
    text = Text(data=mock)
    data, buffer = text.parse()
    assert data == "1"
    assert buffer is None
    assert text.get_more() is None


def test_text_error_type():
    mock = [1, 2, 3]
    with pytest.raises(TypeError):
        Text(data=mock)  # noqa


def test_text_caption():
    mock = "这是一段测试文本"
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == mock
    assert buffer is None
    assert text.get_more()["caption"] == "test"
    # ---------------------------------- float输入 ----------------------------------
    mock = 1.0
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == "1.0"
    assert buffer is None
    assert text.get_more()["caption"] == "test"
    # ---------------------------------- int输入 ----------------------------------
    mock = 1
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == "1"
    assert buffer is None
    assert text.get_more()["caption"] == "test"
    # ---------------------------------- int输入 ----------------------------------
    mock = 1
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == "1"
    assert buffer is None
    assert text.get_more()["caption"] == "test"
    # ---------------------------------- int输入 ----------------------------------
    mock = 1
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == "1"
    assert buffer is None
    assert text.get_more()["caption"] == "test"
    # ---------------------------------- int输入 ----------------------------------
    mock = 1
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == "1"
    assert buffer is None
    assert text.get_more()["caption"] == "test"
    # ---------------------------------- int输入 ----------------------------------
    mock = 1
    text = Text(data=mock, caption="test")
    data, buffer = text.parse()
    assert data == "1"
    assert buffer is None
    assert text.get_more()["caption"] == "test"



"""
@author: cunyue
@file: notification_bark.py
@time: 2025/10/12 13:37
@description: 使用bark通知实验运行情况
"""

import argparse
import random
import time

import swanlab
from swanlab.plugin import BarkCallback

parser = argparse.ArgumentParser(description="测试Bark通知功能")
parser.add_argument('--api-key', type=str, default=None, help="SwanLab的API Key")
parser.add_argument('--host', type=str, default=None, help="SwanLab的服务器地址")
parser.add_argument("--key", type=str, required=True, help="Bark的Key")
parser.add_argument("--url", type=str, default="https://api.day.app/", help="Bark的服务器地址")
args = parser.parse_args()

if args.api_key:
    swanlab.login(api_key=args.api_key, host=args.host)
# 集成BarkCallback
swanlab.register_callbacks([BarkCallback(key=args.key, url=args.url)])

epochs = 50
lr = 0.01
offset = random.random() / 5
# 初始化
swanlab.init(description="测试Bark通知功能", mode="cloud")
swanlab.config.epochs = epochs
swanlab.config.learning_rate = lr
# 模拟训练
for epoch in range(2, swanlab.config.epochs):
    acc = 1 - 2**-epoch - random.random() / epoch - offset
    loss = 2**-epoch + random.random() / epoch + offset
    print(f"epoch={epoch}, accuracy={acc}, loss={loss}")
    swanlab.log({"t/accuracy": acc, "loss": loss})
    time.sleep(0.5)



"""
@author: cunyue
@file: __init__.py
@time: 2024/11/18 15:02
@description: 实验元信息采集
"""

from typing import Tuple, List

from swanlab.package import get_package_version
from swanlab.swanlab_settings import get_settings
from .conda import get_conda
from .cooperation import get_cooperation_info
from .hardware import *
from .requirements import get_requirements
from .runtime import get_runtime_info


def get_metadata(logdir: str = None) -> Tuple[dict, List[HardwareCollector]]:
    """
    采集实验的全部信息
    """
    settings = get_settings()
    # 1. 按照配置采集元信息
    hardware_info, monitor_funcs, runtime_info = {}, [], {}
    if settings.metadata_collect:
        # 1.1 采集软硬件信息
        if settings.collect_hardware:
            hardware_info, monitor_funcs = get_hardware_info()
        # 1.2 如果硬件监控被关闭，则monitor_funcs置空
        if not settings.hardware_monitor:
            monitor_funcs = []
        # 1.3 采集运行时信息
        if settings.collect_runtime:
            runtime_info = get_runtime_info()
    # 2. swanlab官方信息收集
    coop = get_cooperation_info()
    # 2.1 生成基本swanlab信息
    swanlab_info = {
        "version": get_package_version(),
        "_settings": settings.filter_changed_fields(),
        "_coop": coop,
        "_monitor": len(monitor_funcs),
    }
    if logdir is not None:
        swanlab_info["logdir"] = logdir
    # 2.2 如果_coop为空，则删除
    if not coop:
        del swanlab_info["_coop"]
    # 2.3 如果_settings为空，则删除
    if not swanlab_info["_settings"]:
        del swanlab_info["_settings"]
    # 3. 合并所有信息
    metadata = {
        **hardware_info,
        **runtime_info,
        "swanlab": swanlab_info,
    }
    return metadata, monitor_funcs


__all__ = [
    "get_metadata",
    "get_requirements",
    "get_conda",
    "get_cooperation_info",
    "HardwareInfo",
    "HardwareCollector",
]



"""
@author: cunyue
@file: mounter.py
@time: 2025/7/17 13:46
@description: 挂载项目和实验，并在 run_store 中记录相关信息
一般情况下，他与 Porter 一起使用，作为 Porter 的前置操作
"""

from typing import Optional

from swanlab.core_python import get_client, Client
from swanlab.data import namer as N
from swanlab.data.run.config import SwanLabConfig
from swanlab.data.run.metadata.hardware import is_system_key
from swanlab.data.store import RunStore, get_run_store, RemoteMetric


class Mounter:
    """
    Mounter is a context manager that mounts a project and an experiment to the run_store.
    It initializes the run_store with the project and experiment information, and provides methods to execute the mounting process.
    """

    def __init__(self):
        """
        Initializes the Mounter.
        example usage:
        >>> with Mounter() as mounter:
        >>>     mounter.execute()  # create project and experiment, and mount them to run_store
        """
        self._run_store: Optional[RunStore] = None
        self._client: Optional[Client] = None

    @property
    def run_store(self) -> RunStore:
        assert self._run_store is not None, "You can only access run_store inside the context manager."
        return self._run_store

    def __enter__(self):
        try:
            self._client = get_client()
        except ValueError:
            raise ValueError("You must create a client before using Mounter.")
        self._run_store = get_run_store()
        return self

    def execute(self):
        """
        执行挂载
        """
        run_store = self.run_store
        http = self._client
        http.mount_project(run_store.project, run_store.workspace, run_store.visibility)
        # 1. 挂载实验前执行必要的参数生成操作
        exp_count = http.history_exp_count
        run_store.run_name = N.generate_name(exp_count) if run_store.run_name is None else run_store.run_name
        run_store.description = "" if run_store.description is None else run_store.description
        run_store.run_colors = N.generate_colors(exp_count) if run_store.run_colors is None else run_store.run_colors
        run_store.tags = [] if run_store.tags is None else run_store.tags
        # 2. 挂载实验
        try:
            new = http.mount_exp(
                exp_name=run_store.run_name,
                colors=run_store.run_colors,
                description=run_store.description,
                job_type=run_store.job_type,
                group=run_store.group,
                tags=run_store.tags,
                cuid=run_store.run_id,
                must_exist=run_store.resume == 'must',
            )
            run_store.new = new
        except RuntimeError:
            raise RuntimeError(
                "When resume=must, the experiment must exist in project {}. Please check your parameters.".format(
                    http.proj.name
                )
            )
        run_store.run_id = http.exp_id
        # 如果不是新实验，需要获取最新的实验配置和指标数据进行本地同步
        if not new:
            # 1. 解析 config，保存到 run_store.config
            config = http.exp.config
            run_store.config = SwanLabConfig.revert_config(config or {})
            # 2. 获取最新的指标数据，解析为 run_store.metrics
            # 指标总结数据，{log:[{key: '', step: int}] or None, media: [{key: str, step: int}, ...] or None, scalar: [{key: str, step: int}, ...] or None}
            summaries, _ = http.get(
                "/house/metrics/summaries/{}/{}".format(
                    http.exp.root_proj_cuid or http.proj.cuid, http.exp.root_exp_cuid or http.exp_id
                ),
                {"all": True},
            )
            # 设置历史指标条数
            run_store.log_epoch = summaries.get("log", [{}])[0].get("step", 0)
            # 列响应
            columns_resp, _ = http.get("/experiment/{}/column".format(http.exp_id), {"all": True})
            # 列信息, [{error: dict or None, key:str, type:str, class:str}]
            columns = columns_resp.get("list", [])
            # key -> (column_type, column_class, error, latest step)
            metrics: RemoteMetric = {}
            for column in columns:
                # 从列信息中获取指标信息
                key = column["key"]
                column_type = column["type"]
                column_class = column["class"]
                error = column.get("error", None)
                if column_class == "SYSTEM" and not is_system_key(key):
                    # 只记录 sdk 生成的系统指标
                    continue
                # 从总结数据中获取最新的 step
                # 这里需要同时查找 media 和 scalar
                latest_step = None
                for scalar_summary in summaries.get("scalar") or []:
                    if scalar_summary["key"] == key:
                        latest_step = scalar_summary["step"]
                        break
                if latest_step is None:
                    for media_summary in summaries.get("media") or []:
                        if media_summary["key"] == key:
                            latest_step = media_summary["step"]
                            break
                # 极端情况下，总结数据中会不包含此key，此时 latest_step 仍为 None
                # 具体情况就是列创建了，但是没有任何数据，这时候latest_step设置为-1即可，代表可接受所有step
                metrics[key] = (column_type, column_class, error, latest_step or -1)
            run_store.metrics = metrics

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._run_store = None



"""
@author: cunyue
@file: allow.py
@time: 2025/7/3 14:11
@description: 测试 allow 模式的 resume 功能
"""

import random

import swanlab

# 1. 随机生成 run_id
run_id = "".join(random.choices("abcdefghijklmnopqrstuvwxyz0123456789", k=21))


# 2. 初始化第一个实验
# 开启实验并记录
run = swanlab.init(id=run_id, resume='allow')
assert run.id == run_id, "Run ID does not match the expected value"
swanlab.log({"loss": 0.1, "accuracy": 0.9}, step=1)
import time

time.sleep(5)
# 3. 继续实验
run = swanlab.init(id=run_id, resume='allow', reinit=True)
# 上传相同 step 的指标，此时报错
ll = run.log({"loss": 0.1, "accuracy": 0.9}, step=1)
assert ll["loss"].is_error, "Expected loss metric to be in error state due to duplicate step"
assert ll['accuracy'].is_error is not None, "Expected column error to be present for duplicate step"
ll = run.log({"loss": 0.2, "accuracy": 0.8})
assert not ll["loss"].is_error, "Expected loss metric to be logged successfully after reinit"
assert not ll['accuracy'].is_error, "Expected accuracy metric to be logged successfully after reinit"
assert ll['loss'].data == 0.2
assert ll['accuracy'].data == 0.8



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-04-26 17:08:07
@File: swanlab\integration\zhipuai\zhipuai.py
@IDE: vscode
@Description:
    ZhipuAI autolog 集成
"""
import os
import sys
import zhipuai

current_dir = os.path.dirname(os.path.abspath(__file__))
relative_path = os.path.join(current_dir, "..")
sys.path.append(relative_path)

from integration_utils.autologging import AutologAPI

from swanlab.integration.zhipuai.resolver import ZhipuAIClientResponseResolver
import pkg_resources


def get_library_version(library_name):
    try:
        version = pkg_resources.get_distribution(library_name).version
        return version
    except pkg_resources.DistributionNotFound:
        return "Library not found"


# 调用函数并传入库名称
library_name = "zhipuai"
version = get_library_version(library_name)


autolog = AutologAPI(
    name="ZhipuAI",
    symbols=(
        "chat.completions.create",
        "chat.asyncCompletions.create",
    ),
    resolver=ZhipuAIClientResponseResolver(),
    client=zhipuai.ZhipuAI(),
    lib_version=version,
)



"""
@author: cunyue
@file: filter.py
@time: 2025/7/20 16:08
@description: 工具函数
"""

from typing import Optional

from swanlab.data.store import RemoteMetric


# 将逻辑写出来的原因是方便测试
def filter_column(key: str, metrics: RemoteMetric) -> bool:
    """
    筛选函数，检查给定的 key 是否在字典中
    :param key: 要检查的 key
    :param metrics: 可选的存储字典，如果为 None，则默认使用 DataPorter 的 run_store.metrics
    :return: 如果 key 在 store 中，则返回 True，否则返回 False
    """
    # 不存在则保留
    if key not in (metrics or {}):
        return True
    return False


# 将逻辑写出来的原因是方便测试
def filter_metric(key: str, step: int, metrics: RemoteMetric) -> bool:
    """
    筛选函数，检查给定的 key 和 step 是否在字典中
    :param key: 要检查的 key
    :param step: 要检查的 step
    :param metrics: 可选的存储字典，如果为 None，则默认使用 DataPorter 的 run_store.metrics
    :return: 如果 key 在 store 中且 step 大于最新 step，则返回 True，否则返回 False
    """
    metrics = metrics or {}
    if key not in metrics:
        return True
    _, _, _, latest_step = metrics[key]
    if step > latest_step:
        return True
    return False


# 将逻辑写出来的原因是方便测试
def filter_epoch(epoch: int, now_epoch: Optional[int]) -> bool:
    """
    筛选函数，检查给定的 epoch 是否大于 now_epoch
    :param epoch: 要检查的 epoch
    :param now_epoch: 当前的 epoch
    :return: 如果 epoch 大于 now_epoch，则返回 True，否则返回 False
    """
    return epoch > (now_epoch or -1)



from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Record(_message.Message):
    __slots__ = ("message_type", "payload")
    class RecordType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        RECORD_UNKNOWN: _ClassVar[Record.RecordType]
        RECORD_SETUP: _ClassVar[Record.RecordType]
        RECORD_TEARDOWN: _ClassVar[Record.RecordType]
        RECORD_RUNTIME: _ClassVar[Record.RecordType]
        RECORD_COLUMN: _ClassVar[Record.RecordType]
        RECORD_MEDIA: _ClassVar[Record.RecordType]
        RECORD_SCALAR: _ClassVar[Record.RecordType]
        RECORD_LOG: _ClassVar[Record.RecordType]
    RECORD_UNKNOWN: Record.RecordType
    RECORD_SETUP: Record.RecordType
    RECORD_TEARDOWN: Record.RecordType
    RECORD_RUNTIME: Record.RecordType
    RECORD_COLUMN: Record.RecordType
    RECORD_MEDIA: Record.RecordType
    RECORD_SCALAR: Record.RecordType
    RECORD_LOG: Record.RecordType
    MESSAGE_TYPE_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    message_type: Record.RecordType
    payload: _any_pb2.Any
    def __init__(self, message_type: _Optional[_Union[Record.RecordType, str]] = ..., payload: _Optional[_Union[_any_pb2.Any, _Mapping]] = ...) -> None: ...

class SetupRecord(_message.Message):
    __slots__ = ("name", "workspace", "public", "experiment_name", "experiment_description", "experiment_tags", "start_time")
    NAME_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_FIELD_NUMBER: _ClassVar[int]
    EXPERIMENT_NAME_FIELD_NUMBER: _ClassVar[int]
    EXPERIMENT_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXPERIMENT_TAGS_FIELD_NUMBER: _ClassVar[int]
    START_TIME_FIELD_NUMBER: _ClassVar[int]
    name: str
    workspace: str
    public: bool
    experiment_name: str
    experiment_description: str
    experiment_tags: _containers.RepeatedScalarFieldContainer[str]
    start_time: str
    def __init__(self, name: _Optional[str] = ..., workspace: _Optional[str] = ..., public: bool = ..., experiment_name: _Optional[str] = ..., experiment_description: _Optional[str] = ..., experiment_tags: _Optional[_Iterable[str]] = ..., start_time: _Optional[str] = ...) -> None: ...

class TeardownRecord(_message.Message):
    __slots__ = ("state", "error_message", "end_time")
    class StateType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        STATE_UNKNOWN: _ClassVar[TeardownRecord.StateType]
        STATE_SUCCESS: _ClassVar[TeardownRecord.StateType]
        STATE_CRASHED: _ClassVar[TeardownRecord.StateType]
    STATE_UNKNOWN: TeardownRecord.StateType
    STATE_SUCCESS: TeardownRecord.StateType
    STATE_CRASHED: TeardownRecord.StateType
    STATE_FIELD_NUMBER: _ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    END_TIME_FIELD_NUMBER: _ClassVar[int]
    state: TeardownRecord.StateType
    error_message: str
    end_time: str
    def __init__(self, state: _Optional[_Union[TeardownRecord.StateType, str]] = ..., error_message: _Optional[str] = ..., end_time: _Optional[str] = ...) -> None: ...

class RuntimeRecord(_message.Message):
    __slots__ = ("conda_filename", "pip_filename", "config_filename", "metadata_filename")
    CONDA_FILENAME_FIELD_NUMBER: _ClassVar[int]
    PIP_FILENAME_FIELD_NUMBER: _ClassVar[int]
    CONFIG_FILENAME_FIELD_NUMBER: _ClassVar[int]
    METADATA_FILENAME_FIELD_NUMBER: _ClassVar[int]
    conda_filename: str
    pip_filename: str
    config_filename: str
    metadata_filename: str
    def __init__(self, conda_filename: _Optional[str] = ..., pip_filename: _Optional[str] = ..., config_filename: _Optional[str] = ..., metadata_filename: _Optional[str] = ...) -> None: ...

class Range(_message.Message):
    __slots__ = ("minval", "maxval")
    MINVAL_FIELD_NUMBER: _ClassVar[int]
    MAXVAL_FIELD_NUMBER: _ClassVar[int]
    minval: int
    maxval: int
    def __init__(self, minval: _Optional[int] = ..., maxval: _Optional[int] = ...) -> None: ...

class ColumnRecord(_message.Message):
    __slots__ = ("column_key", "column_name", "column_class", "column_type", "column_error", "section_name", "section_type", "chart_name", "chart_index", "chart_y_range", "metric_name", "metric_color")
    class ColumnClass(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        COL_CLASS_CUSTOM: _ClassVar[ColumnRecord.ColumnClass]
        COL_CLASS_SYSTEM: _ClassVar[ColumnRecord.ColumnClass]
    COL_CLASS_CUSTOM: ColumnRecord.ColumnClass
    COL_CLASS_SYSTEM: ColumnRecord.ColumnClass
    class ColumnType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        COL_UNKNOWN: _ClassVar[ColumnRecord.ColumnType]
        COL_FLOAT: _ClassVar[ColumnRecord.ColumnType]
        COL_IMAGE: _ClassVar[ColumnRecord.ColumnType]
        COL_AUDIO: _ClassVar[ColumnRecord.ColumnType]
        COL_TEXT: _ClassVar[ColumnRecord.ColumnType]
        COL_OBJECT3D: _ClassVar[ColumnRecord.ColumnType]
        COL_MOLECULE: _ClassVar[ColumnRecord.ColumnType]
        COL_ECHARTS: _ClassVar[ColumnRecord.ColumnType]
    COL_UNKNOWN: ColumnRecord.ColumnType
    COL_FLOAT: ColumnRecord.ColumnType
    COL_IMAGE: ColumnRecord.ColumnType
    COL_AUDIO: ColumnRecord.ColumnType
    COL_TEXT: ColumnRecord.ColumnType
    COL_OBJECT3D: ColumnRecord.ColumnType
    COL_MOLECULE: ColumnRecord.ColumnType
    COL_ECHARTS: ColumnRecord.ColumnType
    class SectionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        SEC_PUBLIC: _ClassVar[ColumnRecord.SectionType]
        SEC_SYSTEM: _ClassVar[ColumnRecord.SectionType]
        SEC_CUSTOM: _ClassVar[ColumnRecord.SectionType]
        SEC_PINNED: _ClassVar[ColumnRecord.SectionType]
        SEC_HIDDEN: _ClassVar[ColumnRecord.SectionType]
    SEC_PUBLIC: ColumnRecord.SectionType
    SEC_SYSTEM: ColumnRecord.SectionType
    SEC_CUSTOM: ColumnRecord.SectionType
    SEC_PINNED: ColumnRecord.SectionType
    SEC_HIDDEN: ColumnRecord.SectionType
    COLUMN_KEY_FIELD_NUMBER: _ClassVar[int]
    COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    COLUMN_CLASS_FIELD_NUMBER: _ClassVar[int]
    COLUMN_TYPE_FIELD_NUMBER: _ClassVar[int]
    COLUMN_ERROR_FIELD_NUMBER: _ClassVar[int]
    SECTION_NAME_FIELD_NUMBER: _ClassVar[int]
    SECTION_TYPE_FIELD_NUMBER: _ClassVar[int]
    CHART_NAME_FIELD_NUMBER: _ClassVar[int]
    CHART_INDEX_FIELD_NUMBER: _ClassVar[int]
    CHART_Y_RANGE_FIELD_NUMBER: _ClassVar[int]
    METRIC_NAME_FIELD_NUMBER: _ClassVar[int]
    METRIC_COLOR_FIELD_NUMBER: _ClassVar[int]
    column_key: str
    column_name: str
    column_class: ColumnRecord.ColumnClass
    column_type: ColumnRecord.ColumnType
    column_error: _struct_pb2.Struct
    section_name: str
    section_type: ColumnRecord.SectionType
    chart_name: str
    chart_index: str
    chart_y_range: Range
    metric_name: str
    metric_color: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, column_key: _Optional[str] = ..., column_name: _Optional[str] = ..., column_class: _Optional[_Union[ColumnRecord.ColumnClass, str]] = ..., column_type: _Optional[_Union[ColumnRecord.ColumnType, str]] = ..., column_error: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., section_name: _Optional[str] = ..., section_type: _Optional[_Union[ColumnRecord.SectionType, str]] = ..., chart_name: _Optional[str] = ..., chart_index: _Optional[str] = ..., chart_y_range: _Optional[_Union[Range, _Mapping]] = ..., metric_name: _Optional[str] = ..., metric_color: _Optional[_Iterable[str]] = ...) -> None: ...

class MediaRecord(_message.Message):
    __slots__ = ("index", "epoch", "create_time", "key", "key_encoded", "kid", "data", "more")
    INDEX_FIELD_NUMBER: _ClassVar[int]
    EPOCH_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    KEY_ENCODED_FIELD_NUMBER: _ClassVar[int]
    KID_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    MORE_FIELD_NUMBER: _ClassVar[int]
    index: str
    epoch: str
    create_time: str
    key: str
    key_encoded: str
    kid: str
    data: _containers.RepeatedScalarFieldContainer[str]
    more: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, index: _Optional[str] = ..., epoch: _Optional[str] = ..., create_time: _Optional[str] = ..., key: _Optional[str] = ..., key_encoded: _Optional[str] = ..., kid: _Optional[str] = ..., data: _Optional[_Iterable[str]] = ..., more: _Optional[_Iterable[str]] = ...) -> None: ...

class ScalarRecord(_message.Message):
    __slots__ = ("index", "epoch", "create_time", "key", "data")
    INDEX_FIELD_NUMBER: _ClassVar[int]
    EPOCH_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    index: str
    epoch: str
    create_time: str
    key: str
    data: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, index: _Optional[str] = ..., epoch: _Optional[str] = ..., create_time: _Optional[str] = ..., key: _Optional[str] = ..., data: _Optional[_Iterable[float]] = ...) -> None: ...

class LogRecord(_message.Message):
    __slots__ = ("epoch", "level", "message")
    class LogType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        LOG_UNKNOWN: _ClassVar[LogRecord.LogType]
        LOG_INFO: _ClassVar[LogRecord.LogType]
        LOG_WARN: _ClassVar[LogRecord.LogType]
        LOG_ERROR: _ClassVar[LogRecord.LogType]
    LOG_UNKNOWN: LogRecord.LogType
    LOG_INFO: LogRecord.LogType
    LOG_WARN: LogRecord.LogType
    LOG_ERROR: LogRecord.LogType
    EPOCH_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    epoch: str
    level: LogRecord.LogType
    message: str
    def __init__(self, epoch: _Optional[str] = ..., level: _Optional[_Union[LogRecord.LogType, str]] = ..., message: _Optional[str] = ...) -> None: ...



"""
@author: cunyue
@file: webhook.py
@time: 2024/11/18 12:35
@description: 当swanlab初始化完毕时，会调用这个webhook函数，用于通知用户消息
"""

import os

import requests

from swanlab.data.run.metadata import get_cooperation_info
from swanlab.env import SwanLabEnv
from swanlab.log import swanlog


def try_send_webhook():
    """
    尝试发送消息，如果发送失败也不会抛出异常，但是会打印错误信息
    """
    webhook = os.getenv(SwanLabEnv.WEBHOOK.value)
    if not webhook:
        return
    data = get_cooperation_info(swanlab=True)
    try:
        requests.post(webhook, json=data)
    except Exception as e:  # noqa
        swanlog.error(f"Failed to send message to webhook: {e}")



import os
import io


def get_tf_events_tags_type(tf_event_path: str):  
    try:
        import tensorflow as tf
    except ImportError as e:
        raise TypeError(
            "Tensorboard Converter requires tensorflow when process tfevents file. Install with 'pip install tensorflow'."
        )

    """获取TFEvent文件中所有tag的类型，并返回一个字典
    比如{"tag1": "scalar", "tag2": "image", "tag3": "audio", "tag4": "text"}

    Args:
        tf_event_path (_type_): 单个tf_event文件的路径

    Returns:
        _type_: 返回一个字典，键是tag，值是tag的类型
    """
    # 确保路径存在
    assert os.path.exists(tf_event_path), "TFEvent file does not exist"

    # 用来存储所有tag的集合和类型
    tags = {}

    # 遍历所有事件
    for event in tf.compat.v1.train.summary_iterator(tf_event_path):
        for value in event.summary.value:
            if value.tag not in tags:
                if value.HasField("simple_value"):
                    tags[value.tag] = "scalar"
                elif value.HasField("image"):
                    tags[value.tag] = "image"
                elif value.HasField("audio"):
                    tags[value.tag] = "audio"
                elif value.HasField("tensor") and value.tensor.dtype == tf.string:
                    tags[value.tag] = "text"

    return tags


def get_tf_events_tags_data(tf_event_path: str, tags: dict):
    try:
        import tensorflow as tf
    except ImportError as e:
        raise TypeError(
            "Tensorboard Converter requires tensorflow when process tensorboard tfevents file. Install with 'pip install tensorflow'."
        )
    import numpy as np
    from PIL import Image
    
    """获取TFEvent文件中所有tag的数据，并返回一个字典
    比如{"tag1": [(step1, value1), (step2, value2)], "tag2": [(step1, value1), (step2, value2)]}

    Args:
        tf_event_path (str): tf_event文件的路径
        tags (dict): tag的类型字典, 形如{"loss": "scalar", "np_image": "image"}

    Returns:
        _type_: 返回一个字典，键是tag，值是完整的数据, 形如{"loss": [(step1, value1), (step2, value2)], "np_image": [(step1, value1), (step2, value2)]}
    """

    # 用来存储每个tag的数据
    tag_data = {tag: [] for tag in tags}

    # 再次遍历文件，这次是为了提取数据
    for event in tf.compat.v1.train.summary_iterator(tf_event_path):
        # wall_time = datetime.datetime.fromtimestamp(event.wall_time).strftime("%Y-%m-%d %H:%M:%S")
        wall_time = int(event.wall_time)
        for value in event.summary.value:
            if value.tag in tag_data:
                if tags[value.tag] == "scalar":
                    tag_data[value.tag].append((event.step, value.simple_value, wall_time))
                elif tags[value.tag] == "image" and value.HasField("image"):
                    img_str = value.image.encoded_image_string
                    image = Image.open(io.BytesIO(img_str))
                    tag_data[value.tag].append((event.step, image, wall_time))
                elif tags[value.tag] == "audio" and value.HasField("audio"):
                    audio = value.audio
                    audio_np = np.frombuffer(value.audio.encoded_audio_string, dtype=np.int16)
                    sample_rate = audio.sample_rate

                    tag_data[value.tag].append((event.step, [audio_np, int(sample_rate)], wall_time))
                elif tags[value.tag] == "text" and value.HasField("tensor"):
                    text = tf.make_ndarray(value.tensor).item().decode("utf-8")
                    tag_data[value.tag].append((event.step, text, wall_time))

    return tag_data


def find_tfevents(logdir: str, depth: int = 3):
    """查找指定目录下的所有tfevents文件

    Args:
        logdir (str): 日志文件夹路径
        depth (int, optional): 目录深度，默认为3

    Returns:
        Dict: 返回一个字典，键是子目录名，值是该目录下的所有tfevents文件路径列表
    """
    tfevents_dict = {}

    def get_depth(path):
        return path.count(os.sep)

    base_depth = get_depth(logdir)

    for root, dirs, files in os.walk(logdir):
        current_depth = get_depth(root) - base_depth
        if current_depth > depth:
            continue

        for file in files:
            if "tfevents" in file:
                # 使用目录名作为字典的键
                directory_key = os.path.relpath(root, logdir)  # 获取目录的最后一个部分作为键
                if directory_key not in tfevents_dict:
                    tfevents_dict[directory_key] = []
                tfevents_dict[directory_key].append(os.path.join(root, file))

    return tfevents_dict



"""
------example.py------
from swanlab.converter import MLFlowConverter

mlf_converter = MLFlowConverter()
mlf_converter.run(tracking_uri="MLFLOW_TRACKING_URL", experiment=0)
"""
import swanlab
from swanlab.log import swanlog as swl
import time

class MLFLowConverter:
    def __init__(
        self,
        project: str = None,
        workspace: str = None,
        mode: str = "cloud",
        logdir: str = None,
        
        **kwargs,
    ):
        
        self.project = project
        self.workspace = workspace
        self.mode = mode
        self.logdir = logdir
    
    def parse_mlflow_logs(self, tracking_uri:str, experiment:int = None):
    
        try:
            import mlflow
        except ImportError as e:
            raise TypeError(
                "MLFlow Converter requires mlflow. Install with 'pip install mlflow'."
            )

        client = mlflow.MlflowClient(tracking_uri=tracking_uri)

        if experiment is None:
            experiments = client.search_experiments()
        else:
            try:
                ex = client.get_experiment(experiment)
            except mlflow.exceptions.MlflowException:
                ex = client.get_experiment_by_name(experiment)
            if not ex:
                print(f'Error: could not find experiment with id or name "{experiment}"')
                return
            experiments = (ex,)

        # Reverse the order of experiments
        if isinstance(experiments, tuple):
            experiments = experiments[::-1]
        else:
            experiments = list(experiments)[::-1]

        for ex in experiments:
            runs = client.search_runs(ex.experiment_id)
            for run in runs:
                run_id = run.info.run_id
                mlflow_run_name = run.data.tags.get('mlflow.runName')
                
                if swanlab.get_run() is None:
                    swanlab_run = swanlab.init(
                        project=self.project,
                        workspace=self.workspace,
                        experiment_name=mlflow_run_name,
                        mode=self.mode,
                        logdir=self.logdir,
                        description=run.data.tags.get('mlflow.note.content'),
                    )
                else:
                    swanlab_run = swanlab.get_run()
                
                swanlab_run.config.update(dict(
                    mlflow_run_id=run.info.run_id,
                    mlflow_run_name=mlflow_run_name,
                    mlflow_run_description=run.data.tags.get('mlflow.note.content'),
                    mlflow_run_params=run.data.params,
                    mlflow_run_tags={k: v for k, v in run.data.tags.items() if not k.startswith('mlflow')},
                ))
                
                index = 0
                for key in run.data.metrics.keys():
                    for m in client.get_metric_history(run_id, key):
                        swanlab_run.log({m.key: m.value}, step=m.step)
                        index += 1
                        print(f"Index {index}: Metric: {m.key} log finished")
                        # TODO: 等未来上传方案优化后解除延时
                        if index % 5 == 0:
                            time.sleep(1)
                
                swanlab_run.finish()
                
    def run(self, tracking_uri: str, experiment: int = None):
        swl.info("Start converting MLFlow Runs to SwanLab...")
        self.parse_mlflow_logs(
            tracking_uri=tracking_uri,
            experiment=experiment,
        )
        swl.info("Finished converting MLFlow Runs to SwanLab.")


"""
@author:    cunyue
@file:      conda.py
@time:      2025/3/2 00:07
@description: 收集conda信息
"""

import subprocess
from typing import Optional

import yaml


def get_conda() -> Optional[str]:
    """
    获取conda信息，如果不存在则返回None
    Returns: str
    """
    try:
        result = subprocess.run(["conda env export"], shell=True, capture_output=True, text=True, timeout=15)
        if result.returncode != 0:
            return None
        output = result.stdout
        if yaml.safe_load(output):
            return output
    except FileNotFoundError:
        pass
    return None



import os.path
from sys import stdout
from typing import Literal, Union

from rich.status import Status

from .mlflow import sync_mlflow
from .tensorboard import sync_tensorboardX, sync_tensorboard_torch
from .wandb import sync_wandb

__all__ = ["sync_wandb", "sync_tensorboardX", "sync_tensorboard_torch", "sync_mlflow", "sync"]

from .sync_utils import set_run_store
from ..core_python import create_client, get_client
from ..core_python.auth.providers.api_key import code_login
from ..data.porter import DataPorter, Mounter
from ..formatter import check_proj_name_format, check_run_id_format
from ..log import swanlog


def sync(
    dir_path: str,
    workspace: str = None,
    project: str = None,
    id: Union[str, Literal['auto', 'new']] = None,
    api_key: str = None,
    raise_error: bool = True,
):
    """
    Syncs backup files to the cloud. Before syncing, you must log in.
    :param dir_path: The directory path to sync.
    :param id: The ID of the backup to sync. Use cases:
        - None: Equal to 'new'
        - new: Create a new experiment with a new ID.
        - auto: Create (Resume) the experiment with the ID from the backup file.
        - str: Use the specified ID to sync the logs.
    :param workspace: The workspace to sync the logs to. If not specified, it will use the default workspace.
    :param project: The project to sync the logs to. If not specified, it will use the default project.
    :param raise_error: Whether to raise an error if error occurs when syncing.
    :param api_key: If provided, swanlab will sync using this API key. Or you need login first before run this function.
        Attention: If you not provide api-key, you need login every time you run this function.
    """
    # 0. 参数检查
    # 0.1 检查项目名称
    project and check_proj_name_format(project)
    # 0.2 检查实验 ID
    if id is None:
        id = "new"
    if id not in ['new', 'auto']:
        check_run_id_format(id)
    # 1. 根据 api key 登录
    try:
        client = get_client()
    except ValueError:
        client = None
    # api key 存在，则尝试创建客户端
    if api_key:
        client = create_client(code_login(api_key, save_key=False))
    # 2. 开始同步
    try:
        assert os.path.exists(dir_path), f"Directory {dir_path} does not exist."
        stdout.flush()
        with Status("🔁 Syncing...", spinner="dots"):
            with DataPorter().open_for_sync(run_dir=dir_path) as porter:
                proj, exp = porter.parse()
                assert client is not None, "Please log in first before using sync."
                with Mounter() as mounter:
                    run_store = mounter.run_store
                    # 设置运行存储相关信息
                    set_run_store(run_store, proj, exp, project, workspace, id)
                    # 创建实验会话
                    mounter.execute()
                    # 同步
                    porter.synchronize()
        swanlog.info("🚀 Sync completed, View run at ", client.web_exp_url)
    except Exception as e:
        if raise_error:
            raise e
        else:
            return swanlog.error(f"😢 Error syncing: {e}")



"""
@author: cunyue
@file: counter.py
@time: 2025/5/17 17:23
@description: 线程安全计数器，用于记录当前epoch
"""

import threading
from contextlib import ContextDecorator


class AtomicCounter(ContextDecorator):
    """A thread-safe counter that allows free increments within `with` context."""

    def __init__(self, initial_value: int = 0) -> None:
        if initial_value < 0:
            raise ValueError("Counter initial value cannot be negative.")
        self._value = initial_value
        self._lock = threading.Lock()
        self._context_entered = False  # 跟踪是否在上下文管理中

    @property
    def value(self) -> int:
        """Get the current value (thread-safe)."""
        with self._lock:
            return self._value

    def increment(self) -> int:
        """Increment the counter by 1 and return the new value."""
        if not self._context_entered:
            # 不在上下文中时报错
            raise RuntimeError("Counter increment is only allowed within the context manager.")
        else:
            # 在上下文中时直接操作（锁已在__enter__中获取）
            self._value += 1
            return self._value

    def __enter__(self) -> 'AtomicCounter':
        """Enter the context: acquire the lock for batch operations."""
        self._lock.acquire()
        self._context_entered = True
        return self  # 返回self以支持链式调用

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        """Exit the context: release the lock."""
        self._context_entered = False
        self._lock.release()

    def __str__(self) -> str:
        return f"AtomicCounter(value={self.value})"



"""
@author: cunyue
@file: test_metadata_utils.py
@time: 2025/9/9 12:39
@description: 测试工具函数
"""

import os

from swanlab.data.run.metadata.utils import check_env


@check_env('TEST_ENV', default_return='default_value')
def foo():
    return "bar"


def test_check_env():
    """
    测试check_env装饰器功能
    """
    value = foo()
    assert value == "bar"
    os.environ['TEST_ENV'] = 'true'
    value = foo()
    assert value == 'default_value'
    os.environ['TEST_ENV'] = 'TRUE'
    value = foo()
    assert value == 'default_value'
    os.environ['TEST_ENV'] = '0'
    value = foo()
    assert value == "bar"
    del os.environ['TEST_ENV']
    value = foo()
    assert value == "bar"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/4/29 9:40
@File: __init__.py
@IDE: pycharm
@Description:
    SwanLab OpenAPI包
"""

from swanlab.api.main import OpenApi

__all__ = [
    "OpenApi"
]



import pytest

from swanlab.data.run.metadata.hardware.npu.ascend import (
    AscendCollector,
    get_cann_version,
    get_chip_usage,
    get_version,
    map_npu,
)

try:
    npu_version = get_version()
    cann_version = get_cann_version()
    npu_map = map_npu()
    hbm_value: int = 0
    for npu_id in npu_map:
        for chip_id in npu_map[npu_id]:
            chip_info = npu_map[npu_id][chip_id]
            usage = get_chip_usage(npu_id, chip_id)
            hbm_value = int(usage.get("hbm", 0)) if usage else 0
    hbm_value = hbm_value * 1024
except Exception:
    npu_version = None
    cann_version = None
    npu_map = {}
    hbm_value = 0


@pytest.mark.skipif(npu_version is None, reason="No Ascend NPU found")
def test_ascend_collector():
    collector = AscendCollector(npu_map, hbm_value)
    collector()
    assert collector.collect_num == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/11 22:06
@File: pytest_watch.py
@IDE: pycharm
@Description:
    测试cli的watch命令
"""
from swanlab.cli.commands.dashboard.watch import get_free_port
import socket


def test_get_free_port():
    """
    测试get_free_port函数是否能正确获取一个可用端口
    """
    port = get_free_port()
    assert isinstance(port, int)
    assert port == 5092
    # 占用端口
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(('0.0.0.0', 5092))
        port = get_free_port()
        assert isinstance(port, int)
        assert port != 5092



"""
@author: cunyue
@file: model.py
@time: 2025/6/16 14:55
@description: 实验、项目元信息
"""

from typing import Optional


class ProjectInfo:
    def __init__(self, data: dict):
        self.__data = data

    @property
    def cuid(self):
        return self.__data["cuid"]

    @property
    def name(self):
        return self.__data["name"]

    @property
    def history_exp_count(self):
        return self.__data.get('_count', {'experiments': 0})["experiments"]


class ExperimentInfo:

    def __init__(self, data: dict):
        self.__data = data

    @property
    def flag_id(self):
        """
        此实验的标志ID，标志上传时的实验会话
        """
        return self.__data["flagId"]

    @property
    def cuid(self):
        return self.__data["cuid"]

    @property
    def name(self):
        return self.__data["name"]

    @property
    def config(self) -> dict:
        """
        此实验的配置，用于 resume 时同步 config
        """
        return self.__data.get("profile", {}).get("config", {})

    @property
    def root_proj_cuid(self) -> Optional[str]:
        """
        根项目的cuid（上传实验时需要）
        """
        return self.__data.get("rootProId", None)

    @property
    def root_exp_cuid(self) -> Optional[str]:
        """
        根实验的cuid（上传实验时需要）
        """
        return self.__data.get("rootExpId", None)



"""
@author: Puiching-Memory
@file: profiler.py
@time: 2025/6/8 17:47
@description: 保存模型 profiler 日志
"""


def trace_handler(save_dir: str):
    """
    trace_handler 是一个回调函数，用于处理 torch.profiler 的 trace 信息，并将其保存到文件中

    examples
    -------
    >>> activities = [torch.profiler.ProfilerActivity.CPU, torch.profiler.ProfilerActivity.CUDA]
    >>> with torch.profiler.profile(activities=activities,on_trace_ready=trace_handler()) as p:
    """
    from . import swanlog
    import os

    assert os.path.isdir(save_dir), RuntimeError(
        "Run directory not found. Please ensure the run directory is properly set."
    )

    def handler_fn(prof) -> None:
        saved_path = os.path.join(save_dir, 'trace.json')
        if os.path.exists(saved_path):
            swanlog.warning(f"{saved_path} already exists, will be overwritten")
            os.remove(f"{saved_path}")
        else:
            swanlog.info(f"torch.profiler trace is saved to {saved_path}")

        prof.export_chrome_trace(f"{saved_path}")

    return handler_fn



from .notification import *
from .writer import *

__all__ = [
    "EmailCallback",
    "LarkCallback",
    "PrintCallback",
    "CSVWriter",
    "DingTalkCallback",
    "WXWorkCallback",
    "DiscordCallback",
    "SlackCallback",
    "LogdirFileWriter",
    "BarkCallback",
]



"""
@author: xj63
@file: molecule.py
@time: 2025/3/13 13:30
@description: 测试上传分子对象

NOTE 你需要下载下面的文件放到当前文件目录的assets文件夹下，才能运行这个测试
- 3D分子pdb文件: 此文件存放于 https://github.com/SwanHubX/SwanLab/pull/477 ，可通过 https://github.com/user-attachments/files/19605387/molecule.example.pdb.zip 下载
"""

# noinspection PyPackageRequirements
from rdkit import Chem

import swanlab

swanlab.init(project="molecule", public=True)

# from rdkit.Chem.Mol
chem = Chem.MolFromSmiles("CCO")
cco = swanlab.Molecule.from_mol(chem, caption="cco")

# from file path
file = swanlab.Molecule.from_pdb_file("./assets/molecule.example.pdb", caption="file")

# from pdb data
with open("./assets/molecule.example.pdb") as f:
    data = swanlab.Molecule(f.read(), caption="data")

# upload
swanlab.log({"file": file, "data": data, "example": cco})
